@@ -55,7 +55,7 @@ _g_object_unref (gpointer object)
 
 
 void
-_g_object_clear (gpointer p)
+_g_clear_object (gpointer p)
 {
 	gpointer *object_p = (gpointer *) p;
 
@@ -534,6 +534,9 @@ _g_time_val_strftime (GTimeVal   *time_,
 	time_t     secs;
 	struct tm *tm;
 
+	if ((format == NULL) || (*format == '\0'))
+		format = "%Y-%m-%d";
+
 	secs = time_->tv_sec;
 	tm = localtime (&secs);
 	return struct_tm_strftime (tm, format);
@@ -1104,6 +1107,114 @@ _g_str_remove_suffix (const char *s,
 }
 
 
+/* based on glib/glib/gmarkup.c (Copyright 2000, 2003 Red Hat, Inc.)
+ * This version does not escape ' and ''. Needed  because IE does not recognize
+ * &apos; and &quot; */
+void
+_g_string_append_for_html (GString    *str,
+		 	   const char *text,
+		 	   gssize      length)
+{
+	const gchar *p;
+	const gchar *end;
+	gunichar     ch;
+	int          state = 0;
+
+	p = text;
+	end = text + length;
+
+	while (p != end) {
+		const char *next;
+
+		next = g_utf8_next_char (p);
+		ch = g_utf8_get_char (p);
+
+		switch (state) {
+		case 1: /* escaped */
+			if ((ch > 127) || ! g_ascii_isprint ((char) ch))
+				g_string_append_printf (str, "\\&#%d;", ch);
+			else
+				g_string_append_unichar (str, ch);
+			state = 0;
+			break;
+
+		default: /* not escaped */
+			switch (*p) {
+			case '\\':
+				state = 1; /* next character is escaped */
+				break;
+
+			case '&':
+				g_string_append (str, "&amp;");
+				break;
+
+			case '<':
+				g_string_append (str, "&lt;");
+				break;
+
+			case '>':
+				g_string_append (str, "&gt;");
+				break;
+
+			case '\n':
+				g_string_append (str, "<br />");
+				break;
+
+			default:
+				if ((ch > 127) ||  ! g_ascii_isprint ((char)ch))
+					g_string_append_printf (str, "&#%d;", ch);
+				else
+					g_string_append_unichar (str, ch);
+				state = 0;
+				break;
+			}
+		}
+
+		p = next;
+	}
+}
+
+
+char *
+_g_escape_for_html (const char *text,
+		    gssize      length)
+{
+        GString *str;
+
+        g_return_val_if_fail (text != NULL, NULL);
+
+        if (length < 0)
+                length = strlen (text);
+
+        /* prealloc at least as long as original text */
+        str = g_string_sized_new (length);
+        _g_string_append_for_html (str, text, length);
+
+        return g_string_free (str, FALSE);
+}
+
+
+/* Array utils*/
+
+
+char *
+_g_string_array_join (GPtrArray  *array,
+		      const char *separator)
+{
+	GString *s;
+	int      i;
+
+	s = g_string_new ("");
+	for (i = 0; i < array->len; i++) {
+		if ((i > 0) && (separator != NULL))
+			g_string_append (s, separator);
+		g_string_append (s, g_ptr_array_index (array, i));
+	}
+
+	return g_string_free (s, FALSE);
+}
+
+
 /* Regexp utils */
 
 static char **
@@ -1411,6 +1522,22 @@ _g_string_list_dup (GList *string_list)
 }
 
 
+char **
+_g_string_list_to_strv (GList *string_list)
+{
+	char  **strv;
+	GList  *scan;
+	int     i;
+
+	strv = g_new0 (char *, g_list_length (string_list) + 1);
+	for (scan = string_list, i = 0; scan; scan = scan->next)
+		strv[i++] = g_strdup ((char *)scan->data);
+	strv[i++] = NULL;
+
+	return strv;
+}
+
+
 GType
 g_string_list_get_type (void)
 {
@@ -1491,25 +1618,26 @@ _g_uri_get_basename (const char *uri)
 const char *
 _g_uri_get_file_extension (const char *uri)
 {
-	int         len;
-	int         p;
-	const char *ptr = uri;
+	char *p;
 
 	if (uri == NULL)
 		return NULL;
 
-	len = strlen (uri);
-	if (len <= 1)
+	p = strrchr (uri, '.');
+	if (p == NULL)
 		return NULL;
 
-	p = len - 1;
-	while ((p >= 0) && (ptr[p] != '.'))
-		p--;
+	if (p != uri) {
+		char *p2;
 
-	if (p < 0)
-		return NULL;
+		p2 = p - 1;
+		while ((*p2 != '.') && (p2 != uri))
+			p2--;
+		if (strncmp (p2, ".tar.", 5) == 0)
+			p = p2;
+	}
 
-	return uri + p;
+	return p;
 }
 
 
@@ -1617,25 +1745,16 @@ _g_uri_get_parent (const char *uri)
 char *
 _g_uri_remove_extension (const char *uri)
 {
-	int   len;
-	int   p;
-	char *new_path;
+	const char *ext;
 
 	if (uri == NULL)
 		return NULL;
 
-	len = strlen (uri);
-	if (len == 1)
+	ext = _g_uri_get_file_extension (uri);
+	if (ext == NULL)
 		return g_strdup (uri);
-
-	p = len - 1;
-	while ((p > 0) && (uri[p] != '.'))
-		p--;
-	if (p == 0)
-		p = len;
-	new_path = g_strndup (uri, (guint) p);
-
-	return new_path;
+	else
+		return g_strndup (uri, strlen (uri) - strlen (ext));
 }
 
 
@@ -1660,6 +1779,146 @@ _g_build_uri (const char *base, ...)
 }
 
 
+char *
+_g_uri_get_scheme (const char *uri)
+{
+	const char *idx;
+
+	idx = strstr (uri, "://");
+	if (idx == NULL)
+		return NULL;
+	else
+		return g_strndup (uri, (idx - uri) + 3);
+}
+
+
+const char *
+_g_uri_remove_host (const char *uri)
+{
+	const char *idx, *sep;
+
+	if (uri == NULL)
+		return NULL;
+
+	idx = strstr (uri, "://");
+	if (idx == NULL)
+		return uri;
+
+	idx += 3;
+	if (*idx == '\0')
+		return "/";
+
+	sep = strstr (idx, "/");
+	if (sep == NULL)
+		return idx;
+
+	return sep;
+}
+
+
+char *
+_g_uri_get_host (const char *uri)
+{
+	const char *idx;
+
+	idx = strstr (uri, "://");
+	if (idx == NULL)
+		return g_strdup ("file://");
+
+	idx = strstr (idx + 3, "/");
+	if (idx == NULL) {
+		char *scheme;
+
+		scheme = _g_uri_get_scheme (uri);
+		if (scheme == NULL)
+			scheme = g_strdup ("file://");
+		return scheme;
+	}
+
+	return g_strndup (uri, (idx - uri));
+}
+
+
+/* example 1 : uri      = file:///xxx/yyy/zzz/foo
+ *             base     = file:///xxx/www
+ *             return   : ../yyy/zzz/foo
+ *
+ * example 2 : uri      = file:///xxx/yyy/foo
+ *             base     = file:///xxx
+ *             return   : yyy/foo
+ *
+ * example 3 : uri      = smb:///xxx/yyy/foo
+ *             base     = file://hhh/xxx
+ *             return   : smb:///xxx/yyy/foo
+ *
+ * example 4 : uri      = file://hhh/xxx
+ *             base     = file://hhh/xxx
+ *             return   : ./
+ */
+char *
+_g_uri_get_relative_path (const char *uri,
+			  const char *base)
+{
+	char      *uri_host;
+	char      *base_host;
+	gboolean   same_host;
+	char      *source_dir;
+	char     **source_dir_v;
+	char     **base_v;
+	GString   *relative_path;
+	int        i, j;
+
+	if ((uri == NULL) || (base == NULL))
+		return NULL;
+
+	if (strcmp (uri, base) == 0)
+		return g_strdup ("./");
+
+	uri_host = _g_uri_get_host (uri);
+	base_host = _g_uri_get_host (base);
+	same_host = g_str_equal (uri_host, base_host);
+
+	g_free (base_host);
+	g_free (uri_host);
+
+	if (! same_host)
+		return g_strdup (uri);
+
+	source_dir = _g_uri_get_parent (_g_uri_remove_host (uri));
+	source_dir_v = g_strsplit (source_dir, "/", 0);
+	base_v = g_strsplit (_g_uri_remove_host (base), "/", 0);
+
+	relative_path = g_string_new (NULL);
+
+	i = 0;
+	while ((source_dir_v[i] != NULL)
+	       && (base_v[i] != NULL)
+	       && (strcmp (source_dir_v[i], base_v[i]) == 0))
+	{
+		i++;
+	}
+
+	j = i;
+
+	while (base_v[i++] != NULL)
+		g_string_append (relative_path, "../");
+
+	while (source_dir_v[j] != NULL) {
+		g_string_append (relative_path, source_dir_v[j]);
+		g_string_append_c (relative_path, '/');
+		j++;
+	}
+
+	g_string_append (relative_path, _g_uri_get_basename (uri));
+
+	g_strfreev (base_v);
+	g_strfreev (source_dir_v);
+	g_free (source_dir);
+
+	return g_string_free (relative_path, FALSE);
+}
+
+
 /* GIO utils */
 
 
@@ -1735,7 +1994,8 @@ _g_file_get_standard_type (GFile *file)
 		g_error_free (error);
 	}
 
-	g_object_unref (info);
+	if (info != NULL)
+		g_object_unref (info);
 
 	return result;
 }
@@ -1776,6 +2036,54 @@ _g_file_get_destination (GFile *source,
 }
 
 
+GFile *
+_g_file_get_duplicated (GFile *file)
+{
+	GString    *new_uri;
+	char       *uri;
+	char       *uri_noext;
+	GRegex     *regex;
+	GMatchInfo *match_info;
+	GFile      *duplicated;
+
+	new_uri = g_string_new ("");
+	uri = g_file_get_uri (file);
+	uri_noext = _g_uri_remove_extension (uri);
+
+	regex = g_regex_new ("^(.*)%20\\(([0-9]+)\\)$", 0, 0, NULL);
+	g_regex_match (regex, uri_noext, 0, &match_info);
+	if (g_match_info_matches (match_info)) {
+		char    *word;
+		guint64  n;
+
+		word = g_match_info_fetch (match_info, 1);
+		g_string_append (new_uri, word);
+		g_free (word);
+
+		word = g_match_info_fetch (match_info, 2);
+		n = g_ascii_strtoull (word, NULL, 10);
+		g_string_append_printf (new_uri, "%%20(%" G_GUINT64_FORMAT ")", n + 1);
+
+		g_free (word);
+	}
+	else {
+		g_string_append (new_uri, uri_noext);
+		g_string_append (new_uri, "%20(2)");
+	}
+
+	g_string_append (new_uri, _g_uri_get_file_extension (uri));
+	duplicated = g_file_new_for_uri (new_uri->str);
+
+	g_match_info_free (match_info);
+	g_regex_unref (regex);
+	g_free (uri_noext);
+	g_free (uri);
+	g_string_free (new_uri, TRUE);
+
+	return duplicated;
+}
+
+
 GFile *
 _g_file_get_child (GFile *file,
 		   ...)
@@ -2020,15 +2328,8 @@ resolve_symlinks (GFile   *file,
 	}
 
 	path = g_file_get_path (file);
-	if (path == NULL) {
-		char *uri;
-
-		uri = g_file_get_uri (file);
-		*error = g_error_new (G_IO_ERROR, G_IO_ERROR_FAILED, "No local pathname for file: %s.", uri);
-		g_free (uri);
-
-		return NULL;
-	}
+	if (path == NULL)
+		return g_file_dup (file);
 
 	resolved = g_file_new_for_path (G_DIR_SEPARATOR_S);
 
@@ -2088,6 +2389,32 @@ _g_file_resolve_all_symlinks (GFile   *file,
 }
 
 
+gboolean
+_g_file_has_prefix (GFile *file,
+		    GFile *prefix)
+{
+	char     *file_uri;
+	char     *prefix_uri;
+	gboolean  result;
+
+	file_uri = g_file_get_uri (file);
+	prefix_uri = g_file_get_uri (prefix);
+	if (! g_str_has_suffix (prefix_uri, "/")) {
+		char *tmp;
+
+		tmp = g_strconcat (prefix_uri, "/", NULL);
+		g_free (prefix_uri);
+		prefix_uri = tmp;
+	}
+	result = g_str_has_prefix (file_uri, prefix_uri);
+
+	g_free (prefix_uri);
+	g_free (file_uri);
+
+	return result;
+}
+
+
 GFile *
 _g_file_append_prefix (GFile      *file,
 		       const char *prefix)
@@ -2259,11 +2586,11 @@ _g_file_attribute_value_free (_GFileAttributeValue *attr_value)
 	case G_FILE_ATTRIBUTE_TYPE_BYTE_STRING:
 		g_free (attr_value->v.string);
 		break;
-/* FIXME: add if glib >= 2.22
+#if GLIB_CHECK_VERSION(2,22,0)
 	case G_FILE_ATTRIBUTE_TYPE_STRINGV:
 		g_strfreev (attr_value->v.stringv);
 		break;
-*/
+#endif
 	case G_FILE_ATTRIBUTE_TYPE_OBJECT:
 		g_object_unref (attr_value->v.object);
 		break;
@@ -2298,11 +2625,11 @@ _g_file_info_get_value (GFileInfo  *info,
 	case G_FILE_ATTRIBUTE_TYPE_BYTE_STRING:
 		attr_value->v.string = g_strdup ((char *) value);
 		break;
-/* FIXME: add if glib >= 2.22
+#if GLIB_CHECK_VERSION(2,22,0)
 	case G_FILE_ATTRIBUTE_TYPE_STRINGV:
 		attr_value->v.stringv = g_strdupv ((char **) value);
 		break;
-*/
+#endif
 	case G_FILE_ATTRIBUTE_TYPE_BOOLEAN:
 		attr_value->v.boolean = * ((gboolean *) value);
 		break;
@@ -2345,11 +2672,11 @@ _g_file_info_set_value (GFileInfo            *info,
 	case G_FILE_ATTRIBUTE_TYPE_BYTE_STRING:
 		value = attr_value->v.string;
 		break;
-/* FIXME: add if glib >= 2.22
+#if GLIB_CHECK_VERSION(2,22,0)
 	case G_FILE_ATTRIBUTE_TYPE_STRINGV:
 		value = attr_value->v.stringv;
 		break;
-*/
+#endif
 	case G_FILE_ATTRIBUTE_TYPE_BOOLEAN:
 		value = &attr_value->v.boolean;
 		break;
