@@ -25,6 +25,7 @@
 #include <glib/gi18n.h>
 #include "glib-utils.h"
 #include "gth-main.h"
+#include "gth-test-category.h"
 #include "gth-test-simple.h"
 #include "gth-time.h"
 
@@ -265,4 +266,22 @@ gth_main_register_default_tests (void)
 				  "data-type", GTH_TEST_DATA_TYPE_DATE,
 				  "get-data-func", get_original_date_for_test,
 				  NULL);
+	gth_main_register_object (GTH_TYPE_TEST,
+				  "general::title",
+				  GTH_TYPE_TEST_CATEGORY,
+				  "attributes", "general::title",
+				  "display-name", _("Title (embedded)"),
+				  NULL);
+	gth_main_register_object (GTH_TYPE_TEST,
+				  "general::description",
+				  GTH_TYPE_TEST_CATEGORY,
+				  "attributes", "general::description",
+				  "display-name", _("Description (embedded)"),
+				  NULL);
+	gth_main_register_object (GTH_TYPE_TEST,
+				  "general::tags",
+				  GTH_TYPE_TEST_CATEGORY,
+				  "attributes", "general::tags",
+				  "display-name", _("Tag (embedded)"),
+				  NULL);
 }
