@@ -172,7 +172,7 @@ gth_main_init (GthMain *main)
 							g_str_equal,
 							NULL,
 							NULL);
-	main->priv->loaders = g_hash_table_new (g_str_hash, g_str_equal);
+	main->priv->loaders = g_hash_table_new (g_str_hash, (GEqualFunc) g_content_type_equals);
 	main->priv->metadata_category = g_ptr_array_new ();
 	main->priv->metadata_info = g_ptr_array_new ();
 	main->priv->metadata_info_sorted = FALSE;
@@ -256,7 +256,7 @@ gth_main_initialize (void)
 
 	g_set_application_name (_("gthumb"));
 	gtk_window_set_default_icon_name ("gthumb");
-	gtk_icon_theme_append_search_path (gtk_icon_theme_get_default (), GTHUMB_PKGDATADIR G_DIR_SEPARATOR_S "icons");
+	gtk_icon_theme_append_search_path (gtk_icon_theme_get_default (), GTHUMB_ICON_DIR);
 	gtk_about_dialog_set_url_hook (about_dialog_activate_link_cb, NULL, NULL);
 	gtk_about_dialog_set_email_hook (about_dialog_activate_email_cb, NULL, NULL);
 }
@@ -342,20 +342,10 @@ gth_main_get_all_entry_points (void)
 
 	for (scan = gth_main_get_all_file_sources (); scan; scan = scan->next) {
 		GthFileSource *file_source = scan->data;
-		GList         *entry_points;
-		GList         *scan_entry;
-
-		entry_points = gth_file_source_get_entry_points (file_source);
-		for (scan_entry = entry_points; scan_entry; scan_entry = scan_entry->next) {
-			GthFileData *file_data = scan_entry->data;
-
-			list = g_list_prepend (list, g_object_ref (file_data));
-		}
-
-		_g_object_list_unref (entry_points);
+		list = g_list_concat (list, gth_file_source_get_entry_points (file_source));
 	}
 
-	return g_list_reverse (list);
+	return list;
 }
 
 
@@ -417,6 +407,9 @@ gth_main_register_metadata_info (GthMetadataInfo *metadata_info)
 {
 	GthMetadataInfo *info;
 
+	if ((metadata_info->display_name != NULL) && (strstr (metadata_info->display_name, "0x") != NULL))
+		metadata_info->flags = GTH_METADATA_ALLOW_NOWHERE;
+
 	g_static_mutex_lock (&metadata_info_mutex);
 
 	info = gth_metadata_info_dup (metadata_info);
@@ -437,7 +430,8 @@ gth_main_register_metadata_info_v (GthMetadataInfo metadata_info[])
 	g_static_mutex_lock (&metadata_info_mutex);
 
 	for (i = 0; metadata_info[i].id != NULL; i++)
-		g_ptr_array_add (Main->priv->metadata_info, &metadata_info[i]);
+		if ((metadata_info[i].display_name == NULL) || (strstr (metadata_info[i].display_name, "0x") == NULL))
+			g_ptr_array_add (Main->priv->metadata_info, &metadata_info[i]);
 
 	g_static_mutex_unlock (&metadata_info_mutex);
 }
@@ -570,6 +564,9 @@ gth_main_get_metadata_category (const char *id)
 {
 	int i;
 
+	if (id == NULL)
+		return NULL;
+
 	for (i = 0; i < Main->priv->metadata_category->len; i++) {
 		GthMetadataCategory *category;
 
@@ -587,6 +584,9 @@ gth_main_get_metadata_info (const char *id)
 {
 	int i;
 
+	if (id == NULL)
+		return NULL;
+
 	for (i = 0; i < Main->priv->metadata_info->len; i++) {
 		GthMetadataInfo *info;
 
@@ -644,13 +644,21 @@ collect_sort_types (gpointer key,
 }
 
 
+static gboolean
+campare_sort_types (GthFileDataSort *type1,
+		    GthFileDataSort *type2)
+{
+	return g_utf8_collate (type1->display_name, type2->display_name);
+}
+
+
 GList *
 gth_main_get_all_sort_types (void)
 {
 	GList *sort_types = NULL;
 
 	g_hash_table_foreach (Main->priv->sort_types, collect_sort_types, &sort_types);
-	return g_list_reverse (sort_types);
+	return g_list_sort (sort_types, (GCompareFunc) campare_sort_types);
 }
 
 
@@ -718,7 +726,7 @@ gth_main_register_pixbuf_loader (PixbufLoader  loader,
 	va_start (var_args, first_mime_type);
 	mime_type = first_mime_type;
   	while (mime_type != NULL) {
-		g_hash_table_insert (Main->priv->loaders, (gpointer) get_static_string (mime_type), loader);
+		g_hash_table_insert (Main->priv->loaders, (gpointer) get_static_string (g_content_type_from_mime_type (mime_type)), loader);
 		mime_type = va_arg (var_args, const char *);
   	}
 	va_end (var_args);
@@ -1157,10 +1165,9 @@ gth_main_tags_changed (void)
 	gth_user_dir_make_dir_for_file (GTH_DIR_CONFIG, GTHUMB_DIR, TAGS_FILE, NULL);
 	filename = gth_user_dir_get_file (GTH_DIR_CONFIG, GTHUMB_DIR, TAGS_FILE, NULL);
 	gth_tags_file_to_file (Main->priv->tags, filename, NULL);
+	gth_monitor_tags_changed (gth_main_get_default_monitor ());
 
 	g_free (filename);
-
-	gth_monitor_tags_changed (gth_main_get_default_monitor ());
 }
 
 
@@ -1187,22 +1194,34 @@ void
 gth_main_activate_extensions (void)
 {
 	const char *mandatory_extensions[] = {	"file_viewer",
-						"jpeg_utils",
 						NULL };
 	const char *default_extensions[] = {	"bookmarks",
+						"burn_disc",
 						"catalogs",
+						"change_date",
 						"comments",
+						"convert_format",
 						"desktop_background",
-						"exiv2",
+						"edit_metadata",
+						"exiv2_tools",
+						"facebook",
 						"file_manager",
 						"file_tools",
-						"gstreamer",
+						"flicker",
+						"gstreamer_tools",
+						"gstreamer_utils",
 						"image_print",
 						"image_rotation",
 						"image_viewer",
+						"importer",
+						"jpeg_utils",
 						"list_tools",
+						"oauth",
 						"photo_importer",
+						"photobucket",
+						"picasaweb",
 						"pixbuf_savers",
+						"raw_files",
 						"red_eye_removal",
 						"rename_series",
 						"resize_images",
